import re, os, sys, subprocess, copy

try:
    from HTMLParser import HTMLParser
except ImportError:
    from html.parser import HTMLParser

from . import config

emojiRegex = re.compile(r'<span class="emoji emoji(.{1,10})"></span>')
htmlParser = HTMLParser()
try:
    b = u'\u2588'
    sys.stdout.write(b + '\r')
    sys.stdout.flush()
except UnicodeEncodeError:
    BLOCK = 'MM'
else:
    BLOCK = b
friendInfoTemplate = {}
for k in ('UserName', 'City', 'DisplayName', 'PYQuanPin', 'RemarkPYInitial', 'Province',
    'KeyWord', 'RemarkName', 'PYInitial', 'EncryChatRoomId', 'Alias', 'Signature', 
    'NickName', 'RemarkPYQuanPin', 'HeadImgUrl'): friendInfoTemplate[k] = ''
for k in ('UniFriend', 'Sex', 'AppAccountFlag', 'VerifyFlag', 'ChatRoomId', 'HideInputBarFlag',
    'AttrStatus', 'SnsFlag', 'MemberCount', 'OwnerUin', 'ContactFlag', 'Uin',
    'StarFriend', 'Statues'): friendInfoTemplate[k] = 0
friendInfoTemplate['MemberList'] = []

def clear_screen():
    os.system('cls' if config.OS == 'Windows' else 'clear')
def emoji_formatter(d, k):
    # _emoji_deebugger is for bugs about emoji match caused by wechat backstage
    # like :face with tears of joy: will be replaced with :cat face with tears of joy:
    def _emoji_debugger(d, k):
        s = d[k].replace('<span class="emoji emoji1f450"></span',
            '<span class="emoji emoji1f450"></span>') # fix missing bug
        def __fix_miss_match(m):
            return '<span class="emoji emoji%s"></span>' % ({
                '1f63c': '1f601', '1f639': '1f602', '1f63a': '1f603',
                '1f4ab': '1f616', '1f64d': '1f614', '1f63b': '1f60d',
                '1f63d': '1f618', '1f64e': '1f621', '1f63f': '1f622',
                }.get(m.group(1), m.group(1)))
        return emojiRegex.sub(__fix_miss_match, s)
    def _emoji_formatter(m):
        s = m.group(1)
        if len(s) == 6:
            return ('\\U%s\\U%s'%(s[:2].rjust(8, '0'), s[2:].rjust(8, '0'))
                ).encode('utf8').decode('unicode-escape', 'replace')
        elif len(s) == 10:
            return ('\\U%s\\U%s'%(s[:5].rjust(8, '0'), s[5:].rjust(8, '0'))
                ).encode('utf8').decode('unicode-escape', 'replace')
        else:
            return ('\\U%s'%m.group(1).rjust(8, '0')
                ).encode('utf8').decode('unicode-escape', 'replace')
    d[k] = _emoji_debugger(d, k)
    d[k] = emojiRegex.sub(_emoji_formatter, d[k])
def msg_formatter(d, k):
    emoji_formatter(d, k)
    d[k] = d[k].replace('<br/>', '\n')
    d[k]  = htmlParser.unescape(d[k])
def check_file(fileDir):
    try:
        with open(fileDir): pass
        return True
    except:
        return False
def print_qr(fileDir):
    if config.OS == 'Darwin':
        subprocess.call(['open', fileDir])
    elif config.OS == 'Linux':
        subprocess.call(['xdg-open', fileDir])
    else:
        os.startfile(fileDir)
try:
    from PIL import Image
    def print_cmd_qr(fileDir, size = 37, padding = 3,
            white = BLOCK, black = '  ', enableCmdQR = True):
        img     = Image.open(fileDir)
        times   = img.size[0] / (size + padding * 2)
        rgb     = img.convert('RGB')
        try:
            blockCount = int(enableCmdQR)
            assert(0 < abs(blockCount))
        except:
            blockCount = 1
        finally:
            white *= abs(blockCount)
            if blockCount < 0: white, black = black, white
        sys.stdout.write(' '*50 + '\r')
        sys.stdout.flush()
        qr = white * (size + 2) + '\n'
        startPoint = padding + 0.5
        for y in range(size):
            qr += white
            for x in range(size):
                r,g,b = rgb.getpixel(((x + startPoint) * times, (y + startPoint) * times))
                qr += white if r > 127 else black
            qr += white + '\n'
        qr += white * (size + 2) + '\n'
        sys.stdout.write(qr)
except ImportError:
    def print_cmd_qr(fileDir, size = 37, padding = 3,
            white = BLOCK, black = '  ', enableCmdQR = True):
        print('pillow should be installed to use command line QRCode: pip install pillow')
        print_qr(fileDir)
def struct_friend_info(knownInfo):
    member = copy.deepcopy(friendInfoTemplate)
    for k, v in copy.deepcopy(knownInfo).items(): member[k] = v
    return member

def search_dict_list(l, key, value):
    ''' Search a list of dict
        * return dict with specific value & key '''
    for i in l:
        if i.get(key) == value: return i



import os, platform

DIR = os.getcwd()
BASE_URL = 'https://login.weixin.qq.com'
OS = platform.system() #Windows, Linux, Darwin

WELCOME_WORDS = 'Hello World!'
USER_AGENT = u'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.71 Safari/537.36'



TEXT       = 'Text'
MAP        = 'Map'
CARD       = 'Card'
NOTE       = 'Note'
SHARING    = 'Sharing'
PICTURE    = 'Picture'
RECORDING  = 'Recording'
ATTACHMENT = 'Attachment'
VIDEO      = 'Video'
FRIENDS    = 'Friends'



import os, time, copy, Queue

class Storage:
    def __init__(self):
        self.userName          = None
        self.nickName          = None
        self.memberList        = []
        self.mpList            = []
        self.chatroomList      = []
        self.msgList           = Queue.Queue(-1)
        self.lastInputUserName = None
    def dumps(self):
        return {
            'userName'          : self.userName,
            'nickName'          : self.nickName,
            'memberList'        : self.memberList,
            'mpList'            : self.mpList,
            'chatroomList'      : self.chatroomList,
            'lastInputUserName' : self.lastInputUserName, }
    def loads(self, j):
        self.userName          = j.get('userName', None)
        self.nickName          = j.get('nickName', None)
        del self.memberList[:]
        for i in j.get('memberList', []): self.memberList.append(i)
        del self.mpList[:]
        for i in j.get('mpList', []): self.mpList.append(i)
        del self.chatroomList[:]
        for i in j.get('chatroomList', []): self.chatroomList.append(i)
        self.lastInputUserName = j.get('lastInputUserName', None)
    def search_friends(self, name=None, userName=None, remarkName=None, nickName=None,
            wechatAccount=None):
        if (name or userName or remarkName or nickName or wechatAccount) is None:
            return copy.deepcopy(self.memberList[0]) # my own account
        elif userName: # return the only userName match
            for m in self.memberList:
                if m['UserName'] == userName: return copy.deepcopy(m)
        else:
            matchDict = {
                'RemarkName' : remarkName,
                'NickName'   : nickName,
                'Alias'      : wechatAccount, }
            for k in ('RemarkName', 'NickName', 'Alias'):
                if matchDict[k] is None: del matchDict[k]
            if name: # select based on name
                contract = []
                for m in self.memberList:
                    if any([m.get(k) == name for k in ('RemarkName', 'NickName', 'Alias')]):
                        contract.append(m)
            else:
                contract = self.memberList[:]
            if matchDict: # select again based on matchDict
                friendList = []
                for m in contract:
                    if all([m.get(k) == v for k, v in matchDict.items()]):
                        friendList.append(m)
                return copy.deepcopy(friendList)
            else:
                return copy.deepcopy(contract)
    def search_chatrooms(self, name=None, userName=None):
        if userName is not None:
            for m in self.chatroomList:
                if m['UserName'] == userName: return copy.deepcopy(m)
        elif name is not None:
            matchList = []
            for m in self.chatroomList:
                if name in m['NickName']: matchList.append(copy.deepcopy(m))
            return matchList
    def search_mps(self, name=None, userName=None):
        if userName is not None:
            for m in self.mpList:
                if m['UserName'] == userName: return copy.deepcopy(m)
        elif name is not None:
            matchList = []
            for m in self.mpList:
                if name in m['NickName']: matchList.append(copy.deepcopy(m))
            return matchList



import time, Queue

from .client import client
from . import content # this is for creating pyc

__version__ = '1.1.20'

__client = client()
HOT_RELOAD = False
HOT_RELOAD_DIR = 'itchat.pkl'

def auto_login(hotReload=False, statusStorageDir='itchat.pkl',
        enableCmdQR=False, picDir=None):
    global HOT_RELOAD, HOT_RELOAD_DIR
    if hotReload:
        if __client.load_login_status(statusStorageDir): return
        __client.auto_login(enableCmdQR=enableCmdQR, picDir=picDir)
        __client.dump_login_status(statusStorageDir)
        HOT_RELOAD, HOT_RELOAD_DIR = True, statusStorageDir
    else:
        __client.auto_login(enableCmdQR=enableCmdQR, picDir=picDir)
        HOT_RELOAD = False

# The following method are all included in __client.auto_login >>>
def get_QRuuid(): return __client.get_QRuuid()
def get_QR(uuid=None, enableCmdQR=False, picDir=None):
    return __client.get_QR(uuid, enableCmdQR, picDir)
def check_login(uuid=None, picDir=None): return __client.check_login(uuid, picDir)
def web_init(): return __client.web_init()
def show_mobile_login(): return __client.show_mobile_login()
def start_receiving(): return __client.start_receiving()
# <<<

# The following methods are for reload without re-scan the QRCode >>>
def dump_login_status(fileDir='itchat.pkl'): return __client.dump_login_status(fileDir)
def load_login_status(fileDir='itchat.pkl'): return __client.load_login_status(fileDir)
# <<<

# The following methods are for contract dealing >>>
def get_friends(update=False): return __client.get_friends(update)
def search_friends(name=None, userName=None, remarkName=None, nickName=None, wechatAccount=None):
    return __client.storageClass.search_friends(name, userName, remarkName, nickName, wechatAccount)
def set_alias(userName, alias): return __client.set_alias(userName, alias)
def add_friend(userName, status=2, ticket='', userInfo={}): return __client.add_friend(userName, status, ticket, userInfo)
def get_head_img(userName=None, chatroomUserName=None, picDir=None): return __client.get_head_img(userName, chatroomUserName, picDir)
def get_mps(update=False): return __client.get_mps(update)
def search_mps(name=None, userName=None): return __client.storageClass.search_mps(name, userName)
def get_chatrooms(update=False): return __client.get_chatrooms(update)
def search_chatrooms(name=None, userName=None): return __client.storageClass.search_chatrooms(name, userName)
def update_chatroom(groupUserName, detailedMember=False): return __client.update_chatroom(groupUserName, detailedMember)
def create_chatroom(memberList, topic = ''): return __client.create_chatroom(memberList, topic)
def set_chatroom_name(chatroomUserName, name): return __client.set_chatroom_name(chatroomUserName, name)
def delete_member_from_chatroom(chatroomUserName, memberList): return __client.delete_member_from_chatroom(chatroomUserName, memberList)
def add_member_into_chatroom(chatroomUserName, memberList, useInvitation=False): return __client.add_member_into_chatroom(chatroomUserName, memberList, useInvitation)
# <<<

# The following is the tear of age, will be deleted soon
def get_contract(update=False): return __client.get_friends(update)
def get_batch_contract(groupUserName): return __client.update_chatroom(groupUserName)
# <<<

# if toUserName is set to None, msg will be sent to yourself
def send_raw_msg(msgType, content, toUserName): return __client.send_raw_msg(msgType, content, toUserName)
def send_msg(msg = 'Test Message', toUserName = None): return __client.send_msg(msg, toUserName)
def send_file(fileDir, toUserName): return __client.send_file(fileDir, toUserName)
def send_video(fileDir, toUserName): return __client.send_video(fileDir, toUserName)
def send_image(fileDir, toUserName): return __client.send_image(fileDir, toUserName)
def send(msg, toUserName = None):
    if msg is None: return False
    if msg[:5] == '@fil@':
        return __client.send_file(msg[5:], toUserName)
    elif msg[:5] == '@img@':
        return __client.send_image(msg[5:], toUserName)
    elif msg[:5] == '@msg@':
        return __client.send_msg(msg[5:], toUserName)
    elif msg[:5] == '@vid@':
        return __client.send_video(msg[5:], toUserName)
    else:
        return __client.send_msg(msg, toUserName)

# decorations
__functionDict = {'FriendChat': {}, 'GroupChat': {}, 'MpChat': {}}
def configured_reply():
    ''' determine the type of message and reply if its method is defined
        however, I use a strange way to determine whether a msg is from massive platform
        I haven't found a better solution here
        The main problem I'm worrying about is the mismatching of new friends added on phone
        If you have any good idea, pleeeease report an issue. I will be more than grateful. '''
    try:
        msg = __client.msgList.get(timeout=1000)
    except Queue.Empty:
        pass
    else:
        if '@@' in msg['FromUserName']:
            replyFn = __functionDict['GroupChat'].get(msg['Type'])
            if replyFn: send(replyFn(msg), msg.get('FromUserName'))
        elif search_mps(userName=msg['FromUserName']):
            replyFn = __functionDict['MpChat'].get(msg['Type'])
            if replyFn: send(replyFn(msg), msg.get('FromUserName'))
        else:
            replyFn = __functionDict['FriendChat'].get(msg['Type'])
            if replyFn: send(replyFn(msg), msg.get('FromUserName'))

def msg_register(msgType, isFriendChat=False, isGroupChat=False, isMpChat=False):
    ''' a decorator constructor
        return a specific decorator based on information given '''
    if not isinstance(msgType, list): msgType = [msgType]
    def _msg_register(fn):
        for _msgType in msgType:
            if isFriendChat:
                __functionDict['FriendChat'][_msgType] = fn
            if isGroupChat:
                __functionDict['GroupChat'][_msgType] = fn
            if isMpChat:
                __functionDict['MpChat'][_msgType] = fn
            if not any((isFriendChat, isGroupChat, isMpChat)):
                __functionDict['FriendChat'][_msgType] = fn
    return _msg_register

# in-build run
def run(debug=True):
    print('Start auto replying')
    __client.debug = debug
    try:
        while 1: configured_reply()
    except KeyboardInterrupt:
        if HOT_RELOAD: __client.dump_login_status(HOT_RELOAD_DIR)
        print('Bye~')
